#================================================================#
# PLAYERS
#================================================================#

import es, os, playerlib, gamethread
from es import server_var
from etmod_2008.includes.includes import *
from etmod_2008.languages.languages import languages
from etmod_2008.logs.logs import log
from etmod_2008.skills.skills import *
from etmod_2008.classes.classes import classes
from etmod_2008.stats.stats import Stats

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent a players' configuration for the mod '''
class PlayerConfig:
	def __init__(self):
		self.Language = languages["default"]
		self.InGameInfo = True
	def Save(self, filename):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(plPrefPath, filename + ".cfg")), "w")
			f.write("language=" + self.Language.Name + "\n")
			f.write("in_game_info=" + str(int(self.InGameInfo)) + "\n")
			log("'" + filename + "' preferences file successfully saved")
		except:
			log("Couldn't write player preferences to file " + filename)
		finally:
			if f is not None:
				f.close()
	def Load(self, filename):
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(plPrefPath, filename + ".cfg")), "r")
			for line in f:
				parts = str.split(line, "=")
				if len(parts) == 2:
					parts[1] = parts[1].strip("\n")
					parts[1] = parts[1].strip("\r")
					if parts[0] == "language":
						self.Language = languages[parts[1]]
					elif parts[0] == "in_game_info":
						self.InGameInfo = (int(parts[1]) == 1)
			log("'" + str(filename) + "' preferences file successfully loaded")
		except:
			log("Couldn't load player preferences from file " + filename)
		finally:
			if f is not None:
				f.close()
		
''' Used to represent a player's situation while playing '''
class PlayerSituation:
	def __init__(
		self,
		alive = False,
		invAmount = 0,
		page = 1,
		health = 1,
		planted = 0):
		self.Alive = alive
		self.InvAmount = invAmount
		self.Page = page
		self.Planted = planted
		self.Selection = ""
		self.SkillInUse = None
		self.SufferingFrom = None
		
''' Used to represent a skill in progression for a player '''
class PlayerSkill:
	def __init__(self, player, skill):
		self.Player = player
		self.Skill = skill
		self.Amount = 0
		self.Progression = {}
		for p in skill.Parameters.values():
			self.Progression[p.Type] = p.Default
	def Use(self):
		# If he doesn't have one, warn him
		if self.Amount < 1:
			es.tell(self.Player.ID, "You don't have any instance of that skill currently. Buy some to use it.")
			return
			
		# If he's currently using another skill, warn him
		if self.Player.Situation.SkillInUse is not None:
			es.tell(self.Player.ID, "You're currently using skill '" + self.Player.Situation.SkillInUse.Skill.Name + "'")
			return
			
		# Decrease amount first of all
		self.Amount -= 1
		
		# Invisibility
		if self.Skill == skills["invisibility"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			intensity = self.Progression[skillParameterTypes["intensity"]]
			
			# Get class color
			color = Color()
			color.GetFromOther(self.Player.Class.Color)
			color.SetAlpha(255 - intensity)
			
			# Use class color if 'et_use_colors' is activated
			cc = es.getInt("et_use_colors")
			if cc == 0:
				color.SetRed(255)
				color.SetGreen(255)
				color.SetBlue(255)
				
			# Set player transparency
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
				" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
				
			# Info message
			es.tell(self.Player.ID, "Invisibility on for " + str(duration) + " seconds")
			
			# Reset after delay
			#es.delayed(duration, "est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
			#	" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Shield
		elif self.Skill == skills["shield"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			
			# Increase player health and armor
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_SetHealth " + str(self.Player.ID) + " " + str(self.Player.Class.Health + hp))
			es.server.queuecmd("est_SetArmor " + str(self.Player.ID) + " " + str(self.Player.Class.Armor + ap))
			
			# Info message
			es.tell(self.Player.ID, "Shield on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Invincibility
		elif self.Skill == skills["invincibility"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			
			# Set player god like
			self.Player.Situation.SkillInUse = self
			es.server.queuecmd("est_God " + str(self.Player.ID) + " 1")
			
			# Info message
			es.tell(self.Player.ID, "Invincibility on for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, ())
			
		# Poison
		elif self.Skill == skills["poison"]:
			# Get current info
			duration = self.Progression[skillParameterTypes["duration"]]
			hp = self.Progression[skillParameterTypes["health"]]
			ap = self.Progression[skillParameterTypes["armor"]]
			speed = self.Progression[skillParameterTypes["speed"]]
			
			# Get victim
			es.server.queuecmd("es_set et_view_player 0;est_getviewplayer " + str(self.Player.ID) + " et_view_player")
			victim = int(server_var["et_view_player"])
			
			# If he doesn't have a victim in sight, tell him he used one skill for nothing
			if victim == 0:
				es.tell(self.Player.ID, "You failed giving poison to an enemy")
			#	return	# <- UNCOMMENT
			# If he was aiming at a friend, tell him he cannot poison his friends
			#if players[str(victim)].Team == self.Player.Team:	# <- UNCOMMENT
			#	es.tell(self.Player.ID, "Don't try to poison your teammates !")	# <- UNCOMMENT
			#	return	# <- UNCOMMENT
			# If he's got an enemy in sight, poison him
			# Update enemy's situation
			victim = self.Player.ID	# <- REMOVE
			self.Player.Situation.SkillInUse = self
			players[str(victim)].Handicap(self)
			
			# Info message			
			es.tell(self.Player.ID, players[str(victim)].Name + " will suffer from your poison for " + str(duration) + " seconds")
			
			# Reset after delay
			gamethread.delayed(duration, self.Deactivate, (victim))
			
	def Deactivate(self, victim = 0):
		# Get info
		skill = self.Player.Situation.SkillInUse
		
		# If he's using this one then deactivate it
		if skill == self:
			# Reset player's situation
			self.Player.Situation.SkillInUse = None
			
			# For invisibility, reset player color
			if skill.Skill.Name == "invisibility":
				# Get class color
				color = Color()
				color.GetFromOther(self.Player.Class.Color)
				color.SetAlpha(255)
				
				# Use class color if 'et_use_colors' is activated
				cc = es.getInt("et_use_colors")
				if cc == 0:
					color.SetRed(255)
					color.SetGreen(255)
					color.SetBlue(255)
					
				# Reset player color
				es.server.queuecmd("est_SetPlayerColor " + str(self.Player.ID) + " " + str(color.Red) +\
					" " + str(color.Green) + " " + str(color.Blue) + " " + str(color.Alpha) + " 1")
					
				# Info message
				es.tell(self.Player.ID, "Invisibility deactivated")
				
			# For shield, reset player health / armor
			elif skill.Skill.Name == "shield":
				# Reset player's health / armor
				es.server.queuecmd("est_SetHealth " + str(self.Player.ID) + " " + str(self.Player.Class.Health))
				es.server.queuecmd("est_SetArmor " + str(self.Player.ID) + " " + str(self.Player.Class.Armor))
				
				# Info message
				es.tell(self.Player.ID, "Shield deactivated")
				
			# For invincibility, reset player god mode
			elif skill.Skill.Name == "invincibility":
				# Reset player back to normal
				es.server.queuecmd("est_God " + str(self.Player.ID) + " 0")
				
				# Info message
				es.tell(self.Player.ID, "Invincibility deactivated")
				
			# For poison, reset victim's situation
			elif skill.Skill.Name == "poison":
				# Reset victim's situation
				players[str(victim)].Situation.SufferingFrom = None
				
				# Info message
				es.tell(self.Player.ID, players[str(victim)].Name + " is no more suffering from your poison")
		
''' Used to represent ETmod players '''
class Player:
	def __init__(
		self,
		index = -1,
		id = -1,
		steamid = "STEAM_ID_UNDEFINED",
		name = "unamed",
		team = 0,
		curclass = classes["noclass"]):
		self.Index = index
		self.ID = id
		self.SteamID = steamid
		self.Name = name
		self.Team = team
		self.Class = curclass
		self.NextClass = None
		self.Preferences = PlayerConfig()
		self.Stats = Stats()
		self.Skills = {}
		self.Situation = PlayerSituation()
	def IsBot(self):
		return self.SteamID == "BOT"
	def IsPending(self):
		return self.SteamID == "STEAM_ID_PENDING"
	def IsUndefined(self):
		return self.SteamID == "STEAM_ID_UNDEFINED"
	def GiveClassStuff(self, protected = False):
		if self.Team == 2:
			self.Situation.InvAmount = self.Class.InvAmountT
		elif self.Team == 3:
			self.Situation.InvAmount = self.Class.InvAmountCT
		self.Class.GiveStuff(self.ID, self.Team, protected)
	def Heal(self):
		health = es.getplayerprop(self.ID, propIndices["health"])
		newHealth = health + 1
		
		if newHealth > self.Class.Health:
			newHealth = self.Class.Health
			
		es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(newHealth))
		
		return (newHealth != health)
	def Supply(self):
		# PlayerLib stuff
		player = playerlib.getPlayer(self.ID)
		primAmmo = player.get("ammo", "primary")
		secAmmo = player.get("ammo", "secondary")
		
		# New amount
		newPrimAmmo = primAmmo + 1
		newSecAmmo = secAmmo + 1
		
		# Set to max of too much
		if self.Team == 2:
			if newPrimAmmo > self.Class.AmmoPrimaryT:
				newPrimAmmo = self.Class.AmmoPrimaryT
			if newSecAmmo > self.Class.AmmoSecondaryT:
				newSecAmmo = self.Class.AmmoSecondaryT
		elif self.Team == 3:
			if newPrimAmmo > self.Class.AmmoPrimaryCT:
				newPrimAmmo = self.Class.AmmoPrimaryCT
			if newSecAmmo > self.Class.AmmoSecondaryCT:
				newSecAmmo = self.Class.AmmoSecondaryCT
				
		# Set new ammo
		player.set("ammo", ("primary", newPrimAmmo))
		player.set("ammo", ("secondary", newSecAmmo))
		
		# Give inventory item if none left
		invAmount = self.Situation.InvAmount
		if invAmount == 0:
			if self.Team == 2:
				if self.Class.InventoryT is not None:
					es.server.queuecmd("est_Give " + str(self.ID) + " " + self.Class.InventoryT.FullName())
			elif self.Team == 3:
				if self.Class.InventoryCT is not None:
					es.server.queuecmd("est_Give " + str(self.ID) + " " + self.Class.InventoryCT.FullName())
					
		# Increment count
		newInvAmount = invAmount + 1
		
		# Set to max if greater
		if self.Team == 2 and self.Situation.InvAmount > self.Class.InvAmountT:
			newInvAmount = self.Class.InvAmountT
		elif self.Team == 3 and self.Situation.InvAmount > self.Class.InvAmountCT:
			newInvAmount = self.Class.InvAmountCT
			
		# Set to player situation
		self.Situation.InvAmount = newInvAmount
			
		# Return a value to precise if player ammo was updated
		return\
			(newPrimAmmo > primAmmo) or\
			(newSecAmmo > secAmmo) or\
			(newInvAmount > invAmount)
	def Handicap(self, pskill):
		# Get info
		l = self.Preferences.Language
		
		# Update his situation
		self.Situation.SufferingFrom = pskill
		
		# Tell him
		es.tell(self.ID, l.Translate("You've been handicaped by " + pskill.Player.Name + " with skill '" + pskill.Skill.Name + "'"))
		
		# Start the loop
		gamethread.delayed(0, self.HandicapLoop(pskill))
	def HandicapLoop(self, pskill):
		# Get global info
		l = self.Preferences.Language
		speed = pskill.Progression[skillParameterTypes["speed"]]
		
		# If player's situation has been updated and he's no more suffering, tell him and leave
		if self.Situation.SufferingFrom is None:
			es.tell(self.ID, l.Translate("You're no more suffering from '" + pskill.Skill.Name + "'"))
			return
		
		# Poison
		if pskill.Skill.Name == "poison":
			# Get info
			p = playerlib.getPlayer(self.ID)
			health = p.attributes["health"]#es.getplayerprop(self.ID, propIndices["health"])
			armor = p.attributes["armor"]
			hp = pskill.Progression[skillParameterTypes["health"]]
			ap = pskill.Progression[skillParameterTypes["armor"]]
			
			# Reduce his health and armor if he's alive
			if not p.attributes["isdead"]:
				armor -= ap
				if armor < 0: armor = 0			
				es.server.queuecmd("est_SetArmor " + str(self.ID) + " " + str(armor))
				health -= hp
				if health <= 0: es.server.queuecmd("est_Slay " + str(self.ID))
				else : es.server.queuecmd("est_SetHealth " + str(self.ID) + " " + str(health))
			
		# If his situation hasn't changed, call it back based on the skill's speed
		gamethread.delayed(10. / speed, self.HandicapLoop, (pskill))
	def GetPlayerFileName(self):
		return str.replace(self.SteamID, ":", "_")
	def SaveStats(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Stats.Save(self.GetPlayerFileName())
	def LoadStats(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Stats.Load(self.GetPlayerFileName())
	def SavePreferences(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Preferences.Save(self.GetPlayerFileName())
	def LoadPreferences(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		self.Preferences.Load(self.GetPlayerFileName())
	def SaveSkills(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		
		f = None
		try:
			f = open(os.path.join(os.getcwd(), os.path.join(skillsDataPath, self.GetPlayerFileName() + ".skl")), "w")
			for ps in self.Skills.values():
				f.write("[" + ps.Skill.Name + "]\n")
				f.write("amount=" + str(ps.Amount) + "\n")
				for p in ps.Skill.Parameters.keys():
					f.write(p + "=" + str(ps.Progression.get(skillParameterTypes[p], 0)) + "\n")
			log("'" + self.GetPlayerFileName() + "' skills file successfully saved")
		except:
			log("Error while trying to save skills for player " + self.Name)
		finally:
			if f is not None:
				f.close()
	def LoadSkills(self):
		if self.IsBot() or self.IsPending() or self.IsUndefined(): return
		
		f = None
		#try:
		f = open(os.path.join(os.getcwd(), os.path.join(skillsDataPath, self.GetPlayerFileName() + ".skl")), "r")
		self.Skills = {}
		skill = None
		pskill = None
		for line in f:
			line = line.strip("\n")
			line = line.strip("\r")
			if line.startswith("[") and line.endswith("]"):
				skillName = line.strip("[]")
				if skills.has_key(skillName):
					skill = skills[skillName]
					self.Skills[skillName] = PlayerSkill(self, skill)
					pskill = self.Skills[skillName]
				else:
					log("Skill not found for " + self.SteamID + ": " + skillName)
			elif pskill is not None:
				parts = line.split("=")
				if len(parts) == 2:
					var = parts[0]
					val = parts[1]
					if var == "amount":
						pskill.Amount = int(val)
					else:
						pskill.Progression[skillParameterTypes[var]] = int(val)
		log("'" + self.GetPlayerFileName() + "' skills file successfully loaded")
		#except:
		#	log("Error while trying to load skills for player " + self.Name)
		#finally:
		#	if f is not None:
		f.close()
	def __str__(self):
		return self.Name
		
#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
plPrefPath = "cstrike/addons/eventscripts/etmod_2008/players/preferences"
players = {}

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
''' Player joining a class '''
def joinClass(player, targetClass):
	p = player
	c = targetClass

	# Check if the chosen class is active
	if c.Active:
		# Check if he can already use that class
		if	es.getInt("et_restrict_classes") == 0 or\
			hasUnlockedClass(p, c):		
			# Check if there are not and will not be already enough players in that class
			if	es.getInt("et_restrict_players_in_classes") == 0 or\
				(isClassNotFull(c, p.Team) and willClassNotBeFull(c, p.Team)):
				# Check that player hasn't planted a bomb
				if p.Situation.Planted == 0:
					# Check if player doesn't yet have a class
					if p.Class == classes["noclass"]:
						p.Class = c
						p.GiveClassStuff()
					# Prepare his future class if he already has one
					else:
						p.NextClass = c
						es.tell(p.ID, "Next time you'll respawn as a " + c.Name)
				# Otherwise tell him that he can't change class				
				else:
					es.tell(p.ID, "You cannot change class when you have planted a bomb")
			# Otherwise tell him
			elif not (isClassNotFull(c, p.Team) and willClassNotBeFull(c, p.Team)):
				es.tell(p.ID, "That class is currently full. Please choose another one.")
		# Otherwise tell him he has to reach a certain level in a certain class first
		elif not hasUnlockedClass(p, c):
			es.tell(p.ID, "You must reach level " +\
				str(c.UnlockLevel) + " in class " + str(c.UnlockClass) +\
				" before you can play as a " + c.Name)
	# Otherwise tell him that the chosen class is not available
	else:
		es.tell(p.ID, "That class is not available currently")
		
#___________________________________________HELPER FUNCTIONS

# --------------------------- #
# HELPER FUNCTIONS #
# --------------------------- #
''' Check if player has what is required to join a class '''
def hasUnlockedClass(player, targetClass):
	# Check if player has the required level in the unlock class
	c = targetClass
	p = player
	return	c.UnlockClass == "noclass" or\
			c.UnlockClass == "" or\
			(classes.has_key(c.UnlockClass) and\
			p.Stats.Classes.has_key(c.UnlockClass) and\
			p.Stats.GetLevel(c.UnlockClass) >= int(c.UnlockLevel))			
''' Check if there's not too many people in a class already '''
def isClassNotFull(targetClass, team):
	c = targetClass
	
	# Player can join any class if he's alone on server
	if len(players) == 1:
		return True
	
	# Get how many players are in that team and how many are in that class
	inTeam = 0
	inClass = 0
	for p in players.values():
		if p.Team == team:
			inTeam += 1
			if p.Class == c:
				inClass += 1
	
	# Calculate how many people are allowed in that class		
	allowed = float(c.MaxAllowed) / 100 * inTeam
	
	# Return False if there will be too many with him
	return (inClass + 1) <= allowed		
''' Check if there will not be too many people in a class '''
def willClassNotBeFull(targetClass, team):
	c = targetClass
	
	# Get how many players are in that team and how many are in that class
	inTeam = 0
	inClass = 0
	for p in players.values():
		if p.Team == team:
			inTeam += 1
			if p.NextClass == c:
				inClass += 1
	
	# Calculate how many people are allowed in that class		
	allowed = float(c.MaxAllowed) / 100 * inTeam
	
	# Return False if there will be too many with him
	return (inClass + 1) <= allowed
	
#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Log
	log("Players sub-script loaded")	
''' Script unloaded '''
def unload():
	# Log
	log("Players sub-script unloaded")